@@ -1273,7 +1273,8 @@ certauth_try_connect(struct gensio_filter *filter, gensio_time *timeout)
 			gensio_err_to_str(err));
 	    sfilter->pending_err = err;
 	}
-	goto finish_result;
+	if (!sfilter->do_2fa)
+	    goto finish_result;
 
     check_2fa:
 	len = sfilter->len_2fa;
@@ -2541,12 +2542,21 @@ gensio_certauth_filter_config(struct gensio_os_funcs *o,
     return rv;
 }
 
-bool gensio_certauth_filter_config_allow_unencrypted(
+bool
+gensio_certauth_filter_config_allow_unencrypted(
 	     struct gensio_certauth_filter_data *data)
 {
     return data->allow_unencrypted;
 }
 
+bool
+gensio_certauth_filter_config_is_client(
+	     struct gensio_certauth_filter_data *data)
+{
+    return data->is_client;
+}
+
+
 static int
 read_certificate_chain(const char *file, X509 **rcert, STACK_OF(X509) **rca)
 {
