@@ -741,7 +741,9 @@ fd_cleared(struct gensio_iod *iod, void *cb_data)
 	err = fdll->ops->retry_open(fdll->handler_data, &fdll->iod);
 	if (err == GE_INPROGRESS) {
 	    err = fd_setup_handlers(fdll);
-	    if (!err)
+	    if (err)
+		fdll->o->close(&fdll->iod);
+	    else
 		fd_set_state(fdll, FD_IN_OPEN);
 	}
 	if (err) {
