@@ -109,7 +109,7 @@ tracena_new_child(void *acc_data, void **finish_data,
 static int
 tracena_finish_parent(void *acc_data, void *finish_data, struct gensio *io)
 {
-    gensio_set_is_packet(io, true);
+    gensio_set_attr_from_child(io, gensio_get_child(io, 0));
     return 0;
 }
 
