@@ -13,4 +13,7 @@ rb = os.urandom(512)
 io = alloc_io(o, "stdio,cat", chunksize = 64)
 test_dataxfer(io, io, rb)
 io_close(io)
+del io
+del o
+test_shutdown()
 print("  Success!")
