@@ -211,6 +211,9 @@ check_finish(struct ioinfo *ioinfo)
     struct gtinfo *g = gtconn->g;
     struct gensio_os_funcs *o = g->o;
     struct gensio *io;
+    int rv;
+    char data[10];
+    gensiods datasize;
 
     if (!gtconn->close_done || !ogtconn->close_done)
 	return;
@@ -219,11 +222,29 @@ check_finish(struct ioinfo *ioinfo)
     gensio_list_rm(&g->io_list, &ogtconn->link);
 
     io = ioinfo_io(ioinfo);
-    if (io)
+    if (io) {
+	if (!g->err) {
+	    datasize = sizeof(data);
+	    rv = gensio_control(io, GENSIO_CONTROL_DEPTH_FIRST,
+				true, GENSIO_CONTROL_EXIT_CODE,
+				data, &datasize);
+	    if (!rv)
+		g->err = strtol(data, NULL, 0);
+	}
 	gensio_free(io);
+    }
     io = ioinfo_io(oioinfo);
-    if (io)
+    if (io) {
+	if (!g->err) {
+	    datasize = sizeof(data);
+	    rv = gensio_control(io, GENSIO_CONTROL_DEPTH_FIRST,
+				true, GENSIO_CONTROL_EXIT_CODE,
+				data, &datasize);
+	    if (!rv)
+		g->err = strtol(data, NULL, 0);
+	}
 	gensio_free(io);
+    }
 
     o->free(o, gtconn);
     o->free(o, ogtconn);
@@ -523,17 +544,17 @@ add_io(struct gtinfo *g, struct gensio *io, bool open_finished)
     gtconn2->user_io = gtconn1->io;
     gtconn2->io = io;
 
-    if (open_finished)
+    if (open_finished) {
 	ioinfo_set_ready(ioinfo2, gtconn2->io);
-    else
+	if (g->print_laddr)
+	    print_io_addr(io, true);
+	if (g->print_raddr)
+	    print_io_addr(io, false);
+	if (debug)
+	    printf("Connected\r\n");
+    } else {
 	gensio_set_user_data(gtconn2->io, ioinfo2);
-
-    if (g->print_laddr)
-	print_io_addr(io, true);
-    if (g->print_raddr)
-	print_io_addr(io, false);
-    if (debug)
-	printf("Connected\r\n");
+    }
 
     gensio_list_add_tail(&g->io_list, &gtconn1->link);
     gensio_list_add_tail(&g->io_list, &gtconn2->link);
